var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _dec, _dec2, _class;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component } from 'react';
import ReactDOM from 'react-dom';
import Column from './Column';
import { DragSource, DropTarget } from 'react-dnd';
import ItemTypes from './itemTypes';

var rowSource = {
  beginDrag: function beginDrag(props) {
    return {
      id: props.id,
      index: props.index
    };
  }
};

var rowTarget = {
  hover: function hover(props, monitor, component) {

    var dragIndex = monitor.getItem().index;
    var hoverIndex = props.index;

    // Don't replace items with themselves
    if (dragIndex === hoverIndex) {
      return;
    }

    // Determine rectangle on screen
    var hoverBoundingRect = ReactDOM.findDOMNode(component).getBoundingClientRect();

    // Get vertical middle
    var hoverMiddleY = (hoverBoundingRect.bottom - hoverBoundingRect.top) / 2;

    // Determine mouse position
    var clientOffset = monitor.getClientOffset();

    // Get pixels to the top
    var hoverClientY = clientOffset.y - hoverBoundingRect.top;

    // Only perform the move when the mouse has crossed half of the items height
    // When dragging downwards, only move when the cursor is below 50%
    // When dragging upwards, only move when the cursor is above 50%

    // Dragging downwards
    if (dragIndex < hoverIndex && hoverClientY < hoverMiddleY) {
      return;
    }

    // Dragging upwards
    if (dragIndex > hoverIndex && hoverClientY > hoverMiddleY) {
      return;
    }

    // Time to actually perform the action
    props.moveRow(dragIndex, hoverIndex);

    // Note: we're mutating the monitor item here!
    // Generally it's better to avoid mutations,
    // but it's good here for the sake of performance
    // to avoid expensive index searches.
    monitor.getItem().index = hoverIndex;
  }
};

var Row = (_dec = DropTarget(ItemTypes.ROW, rowTarget, function (connect) {
  return {
    connectDropTarget: connect.dropTarget()
  };
}), _dec2 = DragSource(ItemTypes.ROW, rowSource, function (connect, monitor) {
  return {
    connectDragSource: connect.dragSource(),
    isDragging: monitor.isDragging()
  };
}), _dec(_class = _dec2(_class = function (_Component) {
  _inherits(Row, _Component);

  function Row(props) {
    _classCallCheck(this, Row);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.state = {
      isHovering: false
    };
    return _this;
  }

  Row.prototype.handleMouseOver = function handleMouseOver() {
    this.setState({ isHovering: true });
  };

  Row.prototype.handleMouseOut = function handleMouseOut() {
    this.setState({ isHovering: false });
  };

  Row.prototype.render = function render() {
    var _this2 = this;

    var _props = this.props,
        connectDragSource = _props.connectDragSource,
        connectDropTarget = _props.connectDropTarget,
        isDragging = _props.isDragging;

    var opacity = isDragging ? 0 : 1;
    var maxColumns = this.props.row.columns.length === 4 ? true : false;

    return connectDragSource(connectDropTarget(React.createElement(
      'div',
      { className: 'row',
        key: this.props.index,
        onMouseEnter: this.handleMouseOver.bind(this),
        onMouseLeave: this.handleMouseOut.bind(this),
        style: { opacity: opacity } },
      this.state.isHovering && React.createElement(
        'div',
        { className: 'row-actions' },
        React.createElement(
          'a',
          { className: 'row--delete', onClick: this.props.removeRow.bind(this, this.props.index) },
          React.createElement('i', { className: 'fa fa-trash-o', 'aria-hidden': 'true' })
        )
      ),
      this.props.row.columns.map(function (column, i) {
        return React.createElement(Column, _extends({ column: column,
          i: i,
          key: column.code,
          row: _this2.props.row
        }, _this2.props));
      }),
      !maxColumns && React.createElement(
        'button',
        { className: 'column--add', onClick: this.props.addColumn.bind(this, this.props.row, this.props.index) },
        React.createElement('i', { className: 'fa fa-plus' })
      )
    )));
  };

  return Row;
}(Component)) || _class) || _class);


export default Row;